{% extends 'base.html' %}

{% block title %}New Language{% endblock %}

{% block body %}
<h2>Create new Language</h2>

<div id="predefined-lang-container">
  <em>Load from predefined:</em>
  <select id="predefined">
    <option value="-">-</option>
    {% for p in predefined %}
    <option value="{{ p.name }}">{{ p.name }}</option>
    {% endfor %}
  </select>
  <button onclick="location.href='/language/new/' + document.getElementById('predefined').value;">go</button>
</div>

{% include "language/_form.html" %}

{% endblock %}
